@@ -35,8 +35,8 @@ A few obvious properties that are not currently handled realistically:
 - Symlinks and binary files are ignored
 '''
 
-import bisect, collections, json, os, random, time
-from mercurial import cmdutil, context, patch, scmutil, url, util, hg
+import bisect, collections, json, os, random, time, sys
+from mercurial import cmdutil, context, patch, scmutil, util, hg
 from mercurial.i18n import _
 from mercurial.node import nullrev, nullid
 
@@ -231,6 +231,8 @@ def synthesize(ui, repo, descpath, **opts):
     fp.close()
 
     def cdf(l):
+        if not l:
+            return [], []
         vals, probs = zip(*sorted(l, key=lambda x: x[1], reverse=True))
         t = float(sum(probs, 0))
         s, cdfs = 0, []
