@@ -103,16 +103,16 @@ def churn(ui, repo, *pats, **opts):
     Examples::
 
       # display count of changed lines for every committer
-      hg churn -t '{author|email}'
+      hg churn -t "{author|email}"
 
       # display daily activity graph
-      hg churn -f '%H' -s -c
+      hg churn -f "%H" -s -c
 
       # display activity of developers by month
-      hg churn -f '%Y-%m' -s -c
+      hg churn -f "%Y-%m" -s -c
 
       # display count of lines changed in every year
-      hg churn -f '%Y' -s
+      hg churn -f "%Y" -s
 
     It is possible to map alternate email addresses to a main address
     by providing a file using the following format::
@@ -121,6 +121,7 @@ def churn(ui, repo, *pats, **opts):
 
     Such a file may be specified with the --aliases option, otherwise
     a .hgchurn file will be looked for in the working directory root.
+    Aliases will be split from the rightmost "=".
     '''
     def pad(s, l):
         return (s + " " * l)[:l]
@@ -132,7 +133,7 @@ def churn(ui, repo, *pats, **opts):
     if aliases:
         for l in open(aliases, "r"):
             try:
-                alias, actual = l.split('=' in l and '=' or None, 1)
+                alias, actual = l.rsplit('=' in l and '=' or None, 1)
                 amap[alias.strip()] = actual.strip()
             except ValueError:
                 l = l.strip()
@@ -144,8 +145,10 @@ def churn(ui, repo, *pats, **opts):
     if not rate:
         return
 
-    sortkey = ((not opts.get('sort')) and (lambda x: -sum(x[1])) or None)
-    rate.sort(key=sortkey)
+    if opts.get('sort'):
+        rate.sort()
+    else:
+        rate.sort(key=lambda x: (-sum(x[1]), x))
 
     # Be careful not to have a zero maxcount (issue833)
     maxcount = float(max(sum(v) for k, v in rate)) or 1.0
