@@ -24,7 +24,7 @@ except ImportError:
             try:
                 from elementtree.ElementTree import ElementTree, XMLParser
             except ImportError:
-                ElementTree = None
+                pass
 
 class darcs_source(converter_source, commandline):
     def __init__(self, ui, path, rev=None):
@@ -42,7 +42,7 @@ class darcs_source(converter_source, commandline):
             raise util.Abort(_('darcs version 2.1 or newer needed (found %r)') %
                              version)
 
-        if ElementTree is None:
+        if "ElementTree" not in globals():
             raise util.Abort(_("Python ElementTree module is not available"))
 
         self.path = os.path.realpath(path)
@@ -104,9 +104,10 @@ class darcs_source(converter_source, commandline):
         # possible, etree will still raise an exception if any
         # non-printable characters are in the XML changelog.
         parser = XMLParser(encoding='latin-1')
-        fp = self._run(cmd, **kwargs)
-        etree.parse(fp, parser=parser)
-        self.checkexit(fp.close())
+        p = self._run(cmd, **kwargs)
+        etree.parse(p.stdout, parser=parser)
+        p.wait()
+        self.checkexit(p.returncode)
         return etree.getroot()
 
     def format(self):
@@ -139,7 +140,7 @@ class darcs_source(converter_source, commandline):
         # etree can return unicode objects for name, comment, and author,
         # so recode() is used to ensure str objects are emitted.
         return commit(author=self.recode(elt.get('author')),
-                      date=util.datestr(date),
+                      date=util.datestr(date, '%Y-%m-%d %H:%M:%S %1%2'),
                       desc=self.recode(desc).strip(),
                       parents=self.parents[rev])
 
