@@ -41,11 +41,30 @@ Note that the remote Mercurial must also have the largefiles extension
 enabled for this to work.
 
 When you pull a changeset that affects largefiles from a remote
-repository, Mercurial behaves as normal. However, when you update to
-such a revision, any largefiles needed by that revision are downloaded
-and cached (if they have never been downloaded before). This means
-that network access may be required to update to changesets you have
-not previously updated to.
+repository, the largefiles for the changeset will by default not be
+pulled down. However, when you update to such a revision, any
+largefiles needed by that revision are downloaded and cached (if
+they have never been downloaded before). One way to pull largefiles
+when pulling is thus to use --update, which will update your working
+copy to the latest pulled revision (and thereby downloading any new
+largefiles).
+
+If you want to pull largefiles you don't need for update yet, then
+you can use pull with the `--lfrev` option or the :hg:`lfpull` command.
+
+If you know you are pulling from a non-default location and want to
+download all the largefiles that correspond to the new changesets at
+the same time, then you can pull with `--lfrev "pulled()"`.
+
+If you just want to ensure that you will have the largefiles needed to
+merge or rebase with new heads that you are pulling, then you can pull
+with `--lfrev "head(pulled())"` flag to pre-emptively download any largefiles
+that are new in the heads you are pulling.
+
+Keep in mind that network access may now be required to update to
+changesets that you have not previously updated to. The nature of the
+largefiles extension means that updating is no longer guaranteed to
+be a local-only operation.
 
 If you already have large files tracked by Mercurial without the
 largefiles extension, you will need to convert your repository in
@@ -78,16 +97,31 @@ tracked as largefiles::
 
 Files that match one of these patterns will be added as largefiles
 regardless of their size.
+
+The ``largefiles.minsize`` and ``largefiles.patterns`` config options
+will be ignored for any repositories not already containing a
+largefile. To add the first largefile to a repository, you must
+explicitly do so with the --large flag passed to the :hg:`add`
+command.
 '''
 
-from mercurial import commands
+from mercurial import commands, localrepo
 
 import lfcommands
 import reposetup
-import uisetup
+import uisetup as uisetupmod
+
+testedwith = 'internal'
 
 reposetup = reposetup.reposetup
-uisetup = uisetup.uisetup
+
+def featuresetup(ui, supported):
+    # don't die on seeing a repo with the largefiles requirement
+    supported |= set(['largefiles'])
+
+def uisetup(ui):
+    localrepo.localrepository.featuresetupfuncs.add(featuresetup)
+    uisetupmod.uisetup(ui)
 
 commands.norepo += " lfconvert"
 
