@@ -8,11 +8,9 @@
 
 '''setup for largefiles repositories: reposetup'''
 import copy
-import types
 import os
 
-from mercurial import context, error, manifest, match as match_, util, \
-    discovery
+from mercurial import error, manifest, match as match_, util, discovery
 from mercurial import node as node_
 from mercurial.i18n import _
 from mercurial import localrepo
@@ -27,15 +25,6 @@ def reposetup(ui, repo):
     if not repo.local():
         return proto.wirereposetup(ui, repo)
 
-    for name in ('status', 'commitctx', 'commit', 'push'):
-        method = getattr(repo, name)
-        if (isinstance(method, types.FunctionType) and
-            method.func_name == 'wrap'):
-            ui.warn(_('largefiles: repo method %r appears to have already been'
-                    ' wrapped by another extension: '
-                    'largefiles may behave incorrectly\n')
-                    % name)
-
     class lfilesrepo(repo.__class__):
         lfstatus = False
         def status_nolfiles(self, *args, **kwargs):
@@ -102,14 +91,8 @@ def reposetup(ui, repo):
             else:
                 # some calls in this function rely on the old version of status
                 self.lfstatus = False
-                if isinstance(node1, context.changectx):
-                    ctx1 = node1
-                else:
-                    ctx1 = self[node1]
-                if isinstance(node2, context.changectx):
-                    ctx2 = node2
-                else:
-                    ctx2 = self[node2]
+                ctx1 = self[node1]
+                ctx2 = self[node2]
                 working = ctx2.rev() is None
                 parentworking = working and ctx1 == self['.']
 
@@ -125,127 +108,146 @@ def reposetup(ui, repo):
                 if match is None:
                     match = match_.always(self.root, self.getcwd())
 
-                # First check if there were files specified on the
-                # command line.  If there were, and none of them were
-                # largefiles, we should just bail here and let super
-                # handle it -- thus gaining a big performance boost.
-                lfdirstate = lfutil.openlfdirstate(ui, self)
-                if match.files() and not match.anypats():
-                    for f in lfdirstate:
-                        if match(f):
-                            break
-                    else:
-                        return super(lfilesrepo, self).status(node1, node2,
-                                match, listignored, listclean,
-                                listunknown, listsubrepos)
+                wlock = None
+                try:
+                    try:
+                        # updating the dirstate is optional
+                        # so we don't wait on the lock
+                        wlock = self.wlock(False)
+                    except error.LockError:
+                        pass
 
-                # Create a copy of match that matches standins instead
-                # of largefiles.
-                def tostandins(files):
-                    if not working:
-                        return files
-                    newfiles = []
-                    dirstate = self.dirstate
-                    for f in files:
-                        sf = lfutil.standin(f)
-                        if sf in dirstate:
-                            newfiles.append(sf)
-                        elif sf in dirstate.dirs():
-                            # Directory entries could be regular or
-                            # standin, check both
-                            newfiles.extend((f, sf))
+                    # First check if there were files specified on the
+                    # command line.  If there were, and none of them were
+                    # largefiles, we should just bail here and let super
+                    # handle it -- thus gaining a big performance boost.
+                    lfdirstate = lfutil.openlfdirstate(ui, self)
+                    if match.files() and not match.anypats():
+                        for f in lfdirstate:
+                            if match(f):
+                                break
                         else:
-                            newfiles.append(f)
-                    return newfiles
+                            return super(lfilesrepo, self).status(node1, node2,
+                                    match, listignored, listclean,
+                                    listunknown, listsubrepos)
 
-                m = copy.copy(match)
-                m._files = tostandins(m._files)
-
-                result = super(lfilesrepo, self).status(node1, node2, m,
-                    ignored, clean, unknown, listsubrepos)
-                if working:
-
-                    def sfindirstate(f):
-                        sf = lfutil.standin(f)
+                    # Create a copy of match that matches standins instead
+                    # of largefiles.
+                    def tostandins(files):
+                        if not working:
+                            return files
+                        newfiles = []
                         dirstate = self.dirstate
-                        return sf in dirstate or sf in dirstate.dirs()
-
-                    match._files = [f for f in match._files
-                                    if sfindirstate(f)]
-                    # Don't waste time getting the ignored and unknown
-                    # files from lfdirstate
-                    s = lfdirstate.status(match, [], False,
-                            listclean, False)
-                    (unsure, modified, added, removed, missing, _unknown,
-                            _ignored, clean) = s
-                    if parentworking:
-                        for lfile in unsure:
-                            standin = lfutil.standin(lfile)
-                            if standin not in ctx1:
-                                # from second parent
-                                modified.append(lfile)
-                            elif ctx1[standin].data().strip() \
-                                    != lfutil.hashfile(self.wjoin(lfile)):
-                                modified.append(lfile)
+                        for f in files:
+                            sf = lfutil.standin(f)
+                            if sf in dirstate:
+                                newfiles.append(sf)
+                            elif sf in dirstate.dirs():
+                                # Directory entries could be regular or
+                                # standin, check both
+                                newfiles.extend((f, sf))
                             else:
-                                clean.append(lfile)
-                                lfdirstate.normal(lfile)
-                    else:
-                        tocheck = unsure + modified + added + clean
-                        modified, added, clean = [], [], []
+                                newfiles.append(f)
+                        return newfiles
 
-                        for lfile in tocheck:
-                            standin = lfutil.standin(lfile)
-                            if inctx(standin, ctx1):
-                                if ctx1[standin].data().strip() != \
-                                        lfutil.hashfile(self.wjoin(lfile)):
+                    m = copy.copy(match)
+                    m._files = tostandins(m._files)
+
+                    result = super(lfilesrepo, self).status(node1, node2, m,
+                        ignored, clean, unknown, listsubrepos)
+                    if working:
+
+                        def sfindirstate(f):
+                            sf = lfutil.standin(f)
+                            dirstate = self.dirstate
+                            return sf in dirstate or sf in dirstate.dirs()
+
+                        match._files = [f for f in match._files
+                                        if sfindirstate(f)]
+                        # Don't waste time getting the ignored and unknown
+                        # files from lfdirstate
+                        s = lfdirstate.status(match, [], False,
+                                listclean, False)
+                        (unsure, modified, added, removed, missing, _unknown,
+                                _ignored, clean) = s
+                        if parentworking:
+                            for lfile in unsure:
+                                standin = lfutil.standin(lfile)
+                                if standin not in ctx1:
+                                    # from second parent
+                                    modified.append(lfile)
+                                elif ctx1[standin].data().strip() \
+                                        != lfutil.hashfile(self.wjoin(lfile)):
                                     modified.append(lfile)
                                 else:
                                     clean.append(lfile)
-                            else:
-                                added.append(lfile)
+                                    lfdirstate.normal(lfile)
+                        else:
+                            tocheck = unsure + modified + added + clean
+                            modified, added, clean = [], [], []
 
-                    # Standins no longer found in lfdirstate has been removed
-                    for standin in ctx1.manifest():
-                        if not lfutil.isstandin(standin):
-                            continue
-                        lfile = lfutil.splitstandin(standin)
-                        if not match(lfile):
-                            continue
-                        if lfile not in lfdirstate:
-                            removed.append(lfile)
+                            for lfile in tocheck:
+                                standin = lfutil.standin(lfile)
+                                if inctx(standin, ctx1):
+                                    if ctx1[standin].data().strip() != \
+                                            lfutil.hashfile(self.wjoin(lfile)):
+                                        modified.append(lfile)
+                                    else:
+                                        clean.append(lfile)
+                                else:
+                                    added.append(lfile)
 
-                    # Filter result lists
-                    result = list(result)
+                        # Standins no longer found in lfdirstate has been
+                        # removed
+                        for standin in ctx1.manifest():
+                            if not lfutil.isstandin(standin):
+                                continue
+                            lfile = lfutil.splitstandin(standin)
+                            if not match(lfile):
+                                continue
+                            if lfile not in lfdirstate:
+                                removed.append(lfile)
 
-                    # Largefiles are not really removed when they're
-                    # still in the normal dirstate. Likewise, normal
-                    # files are not really removed if they are still in
-                    # lfdirstate. This happens in merges where files
-                    # change type.
-                    removed = [f for f in removed if f not in self.dirstate]
-                    result[2] = [f for f in result[2] if f not in lfdirstate]
+                        # Filter result lists
+                        result = list(result)
 
-                    lfiles = set(lfdirstate._map)
-                    # Unknown files
-                    result[4] = set(result[4]).difference(lfiles)
-                    # Ignored files
-                    result[5] = set(result[5]).difference(lfiles)
-                    # combine normal files and largefiles
-                    normals = [[fn for fn in filelist
-                                if not lfutil.isstandin(fn)]
-                               for filelist in result]
-                    lfiles = (modified, added, removed, missing, [], [], clean)
-                    result = [sorted(list1 + list2)
-                              for (list1, list2) in zip(normals, lfiles)]
-                else:
-                    def toname(f):
-                        if lfutil.isstandin(f):
-                            return lfutil.splitstandin(f)
-                        return f
-                    result = [[toname(f) for f in items] for items in result]
+                        # Largefiles are not really removed when they're
+                        # still in the normal dirstate. Likewise, normal
+                        # files are not really removed if they are still in
+                        # lfdirstate. This happens in merges where files
+                        # change type.
+                        removed = [f for f in removed
+                                   if f not in self.dirstate]
+                        result[2] = [f for f in result[2]
+                                     if f not in lfdirstate]
 
-                lfdirstate.write()
+                        lfiles = set(lfdirstate._map)
+                        # Unknown files
+                        result[4] = set(result[4]).difference(lfiles)
+                        # Ignored files
+                        result[5] = set(result[5]).difference(lfiles)
+                        # combine normal files and largefiles
+                        normals = [[fn for fn in filelist
+                                    if not lfutil.isstandin(fn)]
+                                   for filelist in result]
+                        lfiles = (modified, added, removed, missing, [], [],
+                                  clean)
+                        result = [sorted(list1 + list2)
+                                  for (list1, list2) in zip(normals, lfiles)]
+                    else:
+                        def toname(f):
+                            if lfutil.isstandin(f):
+                                return lfutil.splitstandin(f)
+                            return f
+                        result = [[toname(f) for f in items]
+                                  for items in result]
+
+                    if wlock:
+                        lfdirstate.write()
+
+                finally:
+                    if wlock:
+                        wlock.release()
 
                 if not listunknown:
                     result[4] = []
@@ -405,6 +407,14 @@ def reposetup(ui, repo):
                 wlock.release()
 
         def push(self, remote, force=False, revs=None, newbranch=False):
+            if remote.local():
+                missing = set(self.requirements) - remote.local().supported
+                if missing:
+                    msg = _("required features are not"
+                            " supported in the destination:"
+                            " %s") % (', '.join(sorted(missing)))
+                    raise util.Abort(msg)
+
             outgoing = discovery.findcommonoutgoing(repo, remote.peer(),
                                                     force=force)
             if outgoing.missing:
@@ -446,7 +456,7 @@ def reposetup(ui, repo):
             the largefiles.
             So we do the following:
             For directories that only have largefiles as matches,
-            we explicitly add the largefiles to the matchlist and remove
+            we explicitly add the largefiles to the match list and remove
             the directory.
             In other cases, we leave the match list unmodified.
             '''
