@@ -8,6 +8,8 @@
 from mercurial.i18n import _
 from mercurial import hg, commands, util
 
+testedwith = 'internal'
+
 def share(ui, source, dest=None, noupdate=False):
     """create a new shared repository
 
@@ -15,6 +17,7 @@ def share(ui, source, dest=None, noupdate=False):
     history with another repository.
 
     .. note::
+
        using rollback or extensions that destroy/modify history (mq,
        rebase, etc.) can cause considerable confusion with shared
        clones. In particular, if two shared clones are both updated to
@@ -22,8 +25,7 @@ def share(ui, source, dest=None, noupdate=False):
        with rollback, the other clone will suddenly stop working: all
        operations will fail with "abort: working directory has unknown
        parent". The only known workaround is to use debugsetparents on
-       the broken clone to reset it to a changeset that still exists
-       (e.g. tip).
+       the broken clone to reset it to a changeset that still exists.
     """
 
     return hg.share(ui, source, dest, not noupdate)
@@ -57,7 +59,7 @@ def unshare(ui, repo):
         lock and lock.release()
 
     # update store, spath, sopener and sjoin of repo
-    repo.__init__(ui, repo.root)
+    repo.unfiltered().__init__(repo.baseui, repo.root)
 
 cmdtable = {
     "share":
