@@ -533,7 +533,7 @@ def transplant(ui, repo, *revs, **opts):
     --source/-s specifies another repository to use for selecting changesets,
     just as if it temporarily had been pulled.
     If --branch/-b is specified, these revisions will be used as
-    heads when deciding which changsets to transplant, just as if only
+    heads when deciding which changesets to transplant, just as if only
     these revisions had been pulled.
     If --all/-a is specified, all the revisions up to the heads specified
     with --branch will be transplanted.
@@ -605,6 +605,7 @@ def transplant(ui, repo, *revs, **opts):
     if opts.get('edit'):
         tp.editor = cmdutil.commitforceeditor
 
+    cmdutil.checkunfinished(repo)
     p1, p2 = repo.dirstate.parents()
     if len(repo) > 0 and p1 == revlog.nullid:
         raise util.Abort(_('no revision checked out'))
@@ -683,6 +684,9 @@ def kwtransplanted(repo, ctx, **args):
 def extsetup(ui):
     revset.symbols['transplanted'] = revsettransplanted
     templatekw.keywords['transplanted'] = kwtransplanted
+    cmdutil.unfinishedstates.append(
+        ['series', True, False, _('transplant in progress'),
+         _("use 'hg transplant --continue' or 'hg update' to abort")])
 
 # tell hggettext to extract docstrings from these functions:
 i18nfunctions = [revsettransplanted, kwtransplanted]
