@@ -1,5 +1,3 @@
-#!/usr/bin/env python
-
 from mercurial import demandimport
 demandimport.enable()
 
@@ -10,6 +8,7 @@ def f(obj):
     l = repr(obj)
     l = rsub("0x[0-9a-fA-F]+", "0x?", l)
     l = rsub("from '.*'", "from '?'", l)
+    l = rsub("'<[a-z]*>'", "'<whatever>'", l)
     return l
 
 import os
@@ -36,5 +35,5 @@ print "fred.sub =", f(fred.sub)
 print "fred =", f(fred)
 
 print "re =", f(re)
-print "re.stdout =", f(re.stdout)
+print "re.stderr =", f(re.stderr)
 print "re =", f(re)
